//---------------------------------------------------------------------------
// standard headers
//---------------------------------------------------------------------------

#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <ctype.h>

//---------------------------------------------------------------------------
// main headers
//---------------------------------------------------------------------------

#include "bool.h"
#include "stack.h"
#include "dfmparser.h"

//---------------------------------------------------------------------------
// VCL class
//---------------------------------------------------------------------------

#include "tform.cpp"
#include "tlabel.cpp"
#include "tstatictext.cpp"
#include "tbutton.cpp"
#include "tpanel.cpp"
#include "tgroupbox.cpp"
#include "tradiogroup.cpp"
#include "tcombobox.cpp"
#include "tedit.cpp"
#include "tlistbox.cpp"
#include "tbevel.cpp"
#include "tmemo.cpp"
#include "tbitbtn.cpp"
#include "tspeedbutton.cpp"
#include "timage.cpp"
#include "tcheckbox.cpp"

//---------------------------------------------------------------------------
// constructor
//---------------------------------------------------------------------------

DFMParser::DFMParser (char *FileName)
{
  TextBuf = NULL;
  Stack = NULL;
  Pos = 0;

  ClassGenerate = DIALOG_BOX; // default
//  ClassGenerate = MAIN_WINDOW;

  SaveICO = false;
  SaveGlyph = false;
  SavePicture = false;
  GenerateMain = false;

  // load file
  LoadFile (FileName);

  cppfile = fopen ("test.cpp","w");
  hfile = fopen ("test.h","w");

  MenuDepth = 0;
  IconCount = 0;
  GlyphCount = 0;

  // events
  EventsFunction = NULL;
  MessageType = NULL;
  MessageID = NULL;

}

//---------------------------------------------------------------------------
// delete object
//---------------------------------------------------------------------------

DFMParser::~DFMParser ()
{
  delete [] TextBuf;

  // delete "this"
  //Pop (&Stack) -> Name;
  Pop (Stack);

  // *.h file
  if (ClassGenerate == MAIN_WINDOW)
    {
      fprintf (hfile,"public:\n");
      fprintf (hfile,"  virtual void create();\n");
      fprintf (hfile,"  virtual ~%s();\n",Stack -> Name);
    }

  // reverse all stack
  Reverse (EventsFunction);
  Reverse (MessageType);
  Reverse (MessageID);
  
  // event function
  if (EventsFunction != NULL)
    {
      fprintf (hfile,"public:\n");
      StackElement *TmpEventsFunction = EventsFunction;
      while (TmpEventsFunction != NULL)
        {
          // check for duplicate events function
          if (Search (TmpEventsFunction -> Next,TmpEventsFunction -> Name) == false)
            {
              fprintf (hfile,"  long %s (FXObject*,FXSelector,void*);\n",TmpEventsFunction -> Name);
            }
          TmpEventsFunction = TmpEventsFunction -> Next;
        }
    }

  // Event ID
  if (MessageID != NULL)
    {
      fprintf (hfile,"public:\n");
      fprintf (hfile,"  enum\n");
      fprintf (hfile,"   {\n");

      // special first = FXMainWindow::ID_LAST or FXDialogBox::ID_LAST
      bool first = true;
      StackElement *TmpMessageID = MessageID;

      while (TmpMessageID != NULL)
        {
          // check for duplicate MessageID
          if (Search (TmpMessageID -> Next,TmpMessageID -> Name) == false &&
            // and check for FXWindow::ID_NONE messages
               strcmp (TmpMessageID -> Name,"FXWindow::ID_NONE") != 0)
            {
              if (first == true)
                {
                  if (ClassGenerate == MAIN_WINDOW)
                    fprintf (hfile,"     %s = FXMainWindow::ID_LAST",TmpMessageID -> Name);
                  else
                    fprintf (hfile,"     %s = FXDialogBox::ID_LAST",TmpMessageID -> Name);
                  first = false;
                }
              else
                fprintf (hfile,"     %s",TmpMessageID -> Name);

              // next MessageID
              TmpMessageID = TmpMessageID -> Next;
              if (TmpMessageID != NULL)
                fprintf (hfile,",");
              fprintf (hfile,"\n");
            }
          else
            // find duplicate MessageID - next MessageID
            TmpMessageID = TmpMessageID -> Next;
        }
      fprintf (hfile,"   };\n");
    }

  fprintf (hfile,"};\n");
  fprintf (hfile,"\n");

  if (MessageType != NULL)
    {
      fprintf (hfile,"// Message Map for the %s class\n",Stack -> Name);
      fprintf (hfile,"\n");
      fprintf (hfile,"FXDEFMAP(%s) %sMap[]=\n",Stack -> Name,Stack -> Name);
      fprintf (hfile,"{\n");

      StackElement *TmpEventsFunction = EventsFunction;
      while (MessageType != NULL)
        {
          // FXWindow::ID_NONE message
          if (strcmp (MessageID -> Name,"FXWindow::ID_NONE") == 0)
            fprintf (hfile,"  FXMAPFUNC (%s, %s, %s::%s)",MessageType -> Name,
              MessageID -> Name,Stack -> Name,TmpEventsFunction -> Name);
          else
            // any other message
            fprintf (hfile,"  FXMAPFUNC (%s, %s::%s, %s::%s)",MessageType -> Name,Stack -> Name,
              MessageID -> Name,Stack -> Name,TmpEventsFunction -> Name);
          Pop (MessageType);
          Pop (MessageID);
          TmpEventsFunction = TmpEventsFunction -> Next;

          if (MessageType != NULL)
            fprintf (hfile,",");
          fprintf (hfile,"\n");
        }

      fprintf (hfile,"};\n");
      fprintf (hfile,"\n");
    }
  fprintf (hfile,"// Macro for the %s class hierarchy implementation\n",Stack -> Name);
  fprintf (hfile,"\n");


  // Events

  if (EventsFunction != NULL)

    {

      if (ClassGenerate == MAIN_WINDOW)

        fprintf (hfile,"FXIMPLEMENT(%s,FXMainWindow,%sMap,ARRAYNUMBER(%sMap))\n",

          Stack -> Name,Stack -> Name,Stack -> Name);

      else

        fprintf (hfile,"FXIMPLEMENT(%s,FXDialogBox,%sMap,ARRAYNUMBER(%sMap))\n",
          Stack -> Name,Stack -> Name,Stack -> Name);
    }
  else
    {
      if (ClassGenerate == MAIN_WINDOW)
        fprintf (hfile,"FXIMPLEMENT(%s, FXMainWindow, NULL, 0)\n",Stack -> Name);
      else
        fprintf (hfile,"FXIMPLEMENT(%s, FXDialogBox, NULL, 0)\n",Stack -> Name);
    }
  fprintf (hfile,"\n");
  fprintf (hfile,"#endif // __Test_H__\n");

  // *.cpp file
  if (ClassGenerate == MAIN_WINDOW)
    {
      fprintf (cppfile,"  new FXTooltip (getApp(),0);\n");
      fprintf (cppfile,"}\n");
      fprintf (cppfile,"\n");
      fprintf (cppfile,"void %s::create()\n",Stack -> Name);
      fprintf (cppfile,"{\n");
      fprintf (cppfile,"  FXMainWindow::create();\n");
      fprintf (cppfile,"  show();\n");
      fprintf (cppfile,"}\n");
      fprintf (cppfile,"\n");
      fprintf (cppfile,"%s::~%s()\n",Stack -> Name,Stack -> Name);
      fprintf (cppfile,"{\n");
      fprintf (cppfile,"}\n");
    }
  else
    fprintf (cppfile,"}\n");

  // Event functions
  while (EventsFunction != NULL)
    {
      if (Search (EventsFunction -> Next,EventsFunction -> Name) == false)
        {
          fprintf (cppfile,"\n");
          fprintf (cppfile,"long %s::%s (FXObject* sender,FXSelector sel,void* ptr)\n",Stack -> Name,EventsFunction -> Name);
          fprintf (cppfile,"{\n");

          // ****************************
          //fprintf (cppfile,"  // return target->handle(this,MKUINT(message,SEL_MOTION),ptr);\n");
          //fprintf (cppfile,"  FXMessageBox::information(this,MBOX_OK,\"Test Information\",\"%s\");\n",EventsFunction -> Name);
          fprintf (cppfile,"  setTitle (\"%s\");\n",EventsFunction -> Name);
          fprintf (cppfile,"  return 1;\n");
          fprintf (cppfile,"}\n");
        }
      Pop (EventsFunction);
    }

  // generate main function
  if (GenerateMain == true)
    {
      fprintf (cppfile,"\n");
      fprintf (cppfile,"// Main program\n");
      fprintf (cppfile,"int main(int argc,char *argv[])\n");
      fprintf (cppfile,"{\n");
      fprintf (cppfile,"  // Make application\n");
      fprintf (cppfile,"  FXApp* Application = new FXApp (\"Window\",\"Test\");\n");
      fprintf (cppfile,"\n");
      fprintf (cppfile,"  // Open display\n");
      fprintf (cppfile,"  Application -> init(argc,argv);\n");
      fprintf (cppfile,"\n");
      fprintf (cppfile,"  // Make my own main window\n");
      if (ClassGenerate == MAIN_WINDOW)
        {
          fprintf (cppfile,"  %s* Main = new %s (Application);\n",Stack -> Name,Stack -> Name);
        }
      if (ClassGenerate == DIALOG_BOX)
        {
          fprintf (cppfile,"  FXMainWindow* Main = new FXMainWindow (Application,\"Window\",NULL,NULL,DECOR_ALL,0,0,400,200);\n");
          fprintf (cppfile,"\n");
          fprintf (cppfile,"  // Build a dialog box\n");
          fprintf (cppfile,"  %s *dialog=new %s(Main);\n",Stack -> Name,Stack -> Name);
        }
      fprintf (cppfile,"\n");
      fprintf (cppfile,"  // Resize and display main window\n");
      fprintf (cppfile,"  Application -> create ();\n");
      if (ClassGenerate == DIALOG_BOX)
        {
          fprintf (cppfile,"  Main -> show ();\n");
          fprintf (cppfile,"\n");
          fprintf (cppfile,"  dialog -> show ();\n");
        }
      fprintf (cppfile,"\n");
      fprintf (cppfile,"  // Run\n");
      fprintf (cppfile,"  return Application -> run ();\n");
      fprintf (cppfile,"}\n");
    }


  // close files
  fclose (cppfile);
  fclose (hfile);
  //Pop (&Stack) -> Name;
  Pop (Stack);
}

//---------------------------------------------------------------------------
// LoadFile
//---------------------------------------------------------------------------

void DFMParser::LoadFile (char *FileName)
{
  // open file
  FILE *MCSFile = fopen (FileName,"rb");
  if (MCSFile == NULL)
    throw DFM_PARSER_ERROR_OPEN_FILE;

  // get the file size
  struct stat StatBuf;
  if (fstat (fileno (MCSFile),&StatBuf) == -1)
    throw DFM_PARSER_ERROR_READ_FILE;
  TextSize = StatBuf.st_size;

  // clear bufor 
  if (TextBuf != NULL)
    delete [] TextBuf;
  TextBuf = NULL;

  // read file to TextBuf
  TextBuf = new char [TextSize];
  if (fread (TextBuf,TextSize,1,MCSFile) == -1)
    throw DFM_PARSER_ERROR_READ_FILE;

  // close file
  fclose (MCSFile);
}

//---------------------------------------------------------------------------
// ParseFile
//---------------------------------------------------------------------------

void DFMParser::ParseFile ()
{
  while (Pos < TextSize)
    {
      bool Error;
      char *Text = ReadString (Error);
      if (Error == false)
        {
          // first object - defaul TForm
          if (strcmp (Text,"object") == 0 && Stack == NULL)
            TFormClass ();
          else
          //  object
          //  end
          //  object ...
          // new object with very same hight
          if (strcmp (Text,"object") == 0 && Stack != NULL)
            {
              // delete last object name
              Pop (Stack);

              // read object name and push it into the stack
              bool Error;
              char *ObjectName = ReadString (Error);
              Push (Stack,ObjectName);
              delete [] ObjectName;
              SkipChar (':');

              // get class type
              char *ClassType = ReadString (Error);
              GetClass (ClassType);
              delete [] ClassType;
            }
          else
          //   object
          //   end
          // end ...
          // step to down in stack
          if (strcmp (Text,"end") == 0)
            Pop (Stack);
          else
            // in Text we read a name for new object
            {
              SkipChar (':');

              // read class type
              char *ClassType = ReadString (Error);
              Push (Stack,Text);
              GetClass (ClassType);
              delete [] ClassType;
            }
        }
      delete [] Text;
    }
}

//---------------------------------------------------------------------------
// Skip Char
//---------------------------------------------------------------------------

void inline DFMParser::SkipChar (char SkipChar)
{
  // skip any spaces
  while (TextBuf [Pos] == ' ')
    Pos++;

  // skip selected char
  while (TextBuf [Pos] == SkipChar)
    Pos++;

  // skip any spaces
  while (TextBuf [Pos] == ' ')
    Pos++;
}

//---------------------------------------------------------------------------
// Get Class
//---------------------------------------------------------------------------

void DFMParser::GetClass (char *ClassType)
{
  // TLabel
  if (strcmp ("TLabel",ClassType) == 0)
    TLabelClass ();
  else

  // TStaticText
  if (strcmp ("TStaticText",ClassType) == 0)
    TStaticTextClass ();
  else

  // TPanel
  if (strcmp ("TPanel",ClassType) == 0)
    TPanelClass ();
  else

  // TBevel
  if (strcmp ("TBevel",ClassType) == 0)
    TBevelClass ();
  else

  // TButton
  if (strcmp ("TButton",ClassType) == 0)
    TButtonClass ();
  else

  // TMainMenu
  if (strcmp ("TMainMenu",ClassType) == 0)
    TMainMenuClass ();
  else

  // TMenuItem
  if (strcmp ("TMenuItem",ClassType) == 0)
    TMenuItemClass ();
  else

  // TGroupBox
  if (strcmp ("TGroupBox",ClassType) == 0)
    TGroupBoxClass ();
  else
  
  // TRadioGroup
  if (strcmp ("TRadioGroup",ClassType) == 0)
    TRadioGroupClass ();
  else

  // TComboBox
  if (strcmp ("TComboBox",ClassType) == 0)
    TComboBoxClass ();
  else

  // TListBox
  if (strcmp ("TListBox",ClassType) == 0)
    TListBoxClass ();
  else

  // TEdit
  if (strcmp ("TEdit",ClassType) == 0)
    TEditClass ();
  else

  // TMemo
  if (strcmp ("TMemo",ClassType) == 0)
    TMemoClass ();
  else

  // TBitBtn
  if (strcmp ("TBitBtn",ClassType) == 0)
    TBitBtnClass ();
  else

  // TSpeedButton
  if (strcmp ("TSpeedButton",ClassType) == 0)
    TSpeedButtonClass ();
  else

  // TImage
  if (strcmp ("TImage",ClassType) == 0)
    TImageClass ();
  else

  // TCheckBox
  if (strcmp ("TCheckBox",ClassType) == 0)
    TCheckBoxClass ();

  // unknow class type
  else
    {
      fprintf (cppfile,"  // Class Type: %s, Stack: %s, Stack -> Next: %s\n",ClassType,Stack -> Name,Stack -> Next -> Name);
      bool Error;
      char *Name = ReadString (Error);
      while (strcmp (Name,"object") != 0 && strcmp (Name,"end") != 0)
        {
          delete [] Name;
          Name = ReadString (Error);
        }
      delete [] Name;
    }
}

//---------------------------------------------------------------------------
// Read Digit  0..9, '-',
//---------------------------------------------------------------------------

int DFMParser::ReadDigit (bool &Error)
{
  char DigitBuf [200];

  // read digit
  while (Pos < TextSize)
    {
      if (isdigit (TextBuf [Pos]) || TextBuf [Pos] == '-')
        {
          int DigitBufPos = 0;

          while ((isdigit (TextBuf [Pos]) || TextBuf [Pos] == '-') && Pos < TextSize)
            {
              DigitBuf [DigitBufPos] = TextBuf [Pos];
              Pos++;
              DigitBufPos++;
            }

          DigitBuf [DigitBufPos] = 0;

          int Digit;
          sscanf (DigitBuf,"%i",&Digit);
          Error = false;
          return Digit;
        }
      else

      if (isalpha (TextBuf [Pos]))
        {
          Error = true;
          return 0;
        }
      else
        Pos++;
    }

  Error = true;
  return 0;
}

//---------------------------------------------------------------------------
// Read string a..z, A..Z, 0..9, '_', '.'
//---------------------------------------------------------------------------

char *DFMParser::ReadString (bool &Error)
{
  char *StringBuf = new char [200];

  while (Pos < TextSize)
    {
      if (isdigit (TextBuf [Pos]) || isalpha (TextBuf [Pos]) ||
          TextBuf [Pos] == '_' || TextBuf [Pos] == '.')
        {
          int StringBufPos = 0;

          while ((isdigit (TextBuf [Pos]) || isalpha (TextBuf [Pos]) ||
                  TextBuf [Pos] == '_' || TextBuf [Pos] == '.') && Pos < TextSize)
            {
              StringBuf [StringBufPos] = TextBuf [Pos];
              Pos++;
              StringBufPos++;
            }

          StringBuf [StringBufPos] = 0;

          Error = false;
          return StringBuf;
        }
      else

      if (isalpha (TextBuf [Pos]))
        {
          Error = true;
          delete [] StringBuf;
          return NULL;
        }
      else
        Pos++;
    }

  Error = true;
  delete [] StringBuf;
  return NULL;
}

//---------------------------------------------------------------------------
// Read Quotation ' string '
//---------------------------------------------------------------------------

char *DFMParser::ReadQuotation (bool &Error)
{
  char *Buf = new char [256];

  while (Pos < TextSize)
    {
      if (TextBuf [Pos] == '\'')
        {
          Pos++;
          int BufPos = 0;
          while (TextBuf [Pos] != '\'' && Pos < TextSize)
            {
              Buf [BufPos] = TextBuf [Pos];
              BufPos++;
              Pos++;
            }
          Buf [BufPos] = 0;
          Error = false;
          return Buf;
        }
      else

      // ' ' '\n' '\r'
      if (TextBuf [Pos] != ' ' && TextBuf [Pos] != '\n' && TextBuf [Pos] != '\r')
        break;

      else
        Pos++;
    }

  delete [] Buf;
  Error = true;
  return NULL;
}

//---------------------------------------------------------------------------
// Read Icon
//---------------------------------------------------------------------------

char *DFMParser::ReadIcon (bool &Error)
{
  // search for '{'
  while (TextBuf [Pos] != '{')
    Pos++;

  // count length
  int Len = 0;
  int TmpPos = Pos + 1;
  while (TextBuf [TmpPos] != '}' && TmpPos < TextSize)
    {
      if ((TextBuf [TmpPos] >= '0' && TextBuf [TmpPos] <= '9') ||
          (TextBuf [TmpPos] >= 'A' && TextBuf [TmpPos] <= 'F'))
        Len++;
      TmpPos++;
    }

  // read
  char *Buf = new char [Len + 1];
  int BufPos = 0;
  TmpPos = Pos + 1;
  while (TextBuf [TmpPos] != '}' && TmpPos < TextSize)
    {
      if ((TextBuf [TmpPos] >= '0' && TextBuf [TmpPos] <= '9') ||
          (TextBuf [TmpPos] >= 'A' && TextBuf [TmpPos] <= 'F'))
        {
          Buf [BufPos] = TextBuf [TmpPos];
          BufPos++;
        }
      TmpPos++;
    }
  Buf [BufPos] = 0;

  Error = false;
  return Buf;
}

//---------------------------------------------------------------------------
// read string property
//---------------------------------------------------------------------------

void inline DFMParser::ReadStringProperty (char *Name,char *Property,char *&Value)
{
  if (strcmp (Name,Property) == 0)
    {
      bool Error = false;
      SkipChar ('=');
      Value = ReadString (Error);
      if (Error == true)
        Value = NULL;
    }
}

//---------------------------------------------------------------------------
// read digit property
//---------------------------------------------------------------------------

void inline DFMParser::ReadDigitProperty (char *Name,char *Property,int &Value)
{
  if (strcmp (Name,Property) == 0)
    {
      bool Error = false;
      SkipChar ('=');
      Value = ReadDigit (Error);
      if (Error == true)
        Value = 0;
    }
}

//---------------------------------------------------------------------------
// read quotation property
//---------------------------------------------------------------------------

void inline DFMParser::ReadQuotationProperty (char *Name,char *Property,char *&Value)
{
  if (strcmp (Name,Property) == 0)
    {
      bool Error = false;
      SkipChar ('=');
      Value = ReadQuotation (Error);
      if (Error == true)
        Value = NULL;
    }
}

//---------------------------------------------------------------------------
// read 'set' property
//---------------------------------------------------------------------------

void inline DFMParser::ReadSetProperty (char *Name,char *Property,StackElement *&Value)
{
  if (strcmp (Name,Property) == 0)
    {
      SkipChar ('=');
      SkipChar ('[');
      char *Element = NULL;
      while (TextBuf [Pos] != ']')
        {
          bool Error = false;
          Element = ReadString (Error);
          if (Error == false)
            Push (Value,Element);
          delete [] Element;
        }
    }
}

//---------------------------------------------------------------------------
// Get VCL Color
//---------------------------------------------------------------------------

char *DFMParser::GetColor (char *ColorName)
{
/*
  /// Obtain default colors
  FXColor getBorderColor() const { return borderColor; }
  FXColor getBaseColor() const { return baseColor; }
  FXColor getHiliteColor() const { return hiliteColor; }
  FXColor getShadowColor() const { return shadowColor; }
  FXColor getBackColor() const { return backColor; }
  FXColor getForeColor() const { return foreColor; }
  FXColor getSelforeColor() const { return selforeColor; }
  FXColor getSelbackColor() const { return selbackColor; }
*/
/*
  FXColor          borderColor;         // Border color
  FXColor          baseColor;           // Background color of GUI controls
  FXColor          hiliteColor;         // Highlight color of GUI controls
  FXColor          shadowColor;         // Shadow color of GUI controls
  FXColor          backColor;           // Background color
  FXColor          foreColor;           // Foreground color
  FXColor          selforeColor;        // Select foreground color
  FXColor          selbackColor;        // Select background color
*/
/*
static const TColor clScrollBar = 0x80000000;
static const TColor clBackground = 0x80000001;
static const TColor clActiveCaption = 0x80000002;
static const TColor clInactiveCaption = 0x80000003;
static const TColor clMenu = 0x80000004;
static const TColor clWindow = 0x80000005;
static const TColor clWindowFrame = 0x80000006;
static const TColor clMenuText = 0x80000007;
static const TColor clWindowText = 0x80000008;
static const TColor clCaptionText = 0x80000009;
static const TColor clActiveBorder = 0x8000000a;
static const TColor clInactiveBorder = 0x8000000b;
static const TColor clAppWorkSpace = 0x8000000c;
static const TColor clHighlight = 0x8000000d;
static const TColor clHighlightText = 0x8000000e;
// static const TColor clBtnFace = 0x8000000f;
static const TColor clBtnShadow = 0x80000010;
static const TColor clGrayText = 0x80000011;
static const TColor clBtnText = 0x80000012;
static const TColor clInactiveCaptionText = 0x80000013;
static const TColor clBtnHighlight = 0x80000014;
static const TColor cl3DDkShadow = 0x80000015;
static const TColor cl3DLight = 0x80000016;
static const TColor clInfoText = 0x80000017;
static const TColor clInfoBk = 0x80000018;
static const TColor clNone = 0x1fffffff;
static const TColor clDefault = 0x20000000;
*/

  // clBtnFace
  if (strcmp (ColorName,"clBtnFace") == 0)
    return "";

  // Black
  if (strcmp (ColorName,"clBlack") == 0)
    return "0x0";

  // Maroon
  if (strcmp (ColorName,"clMaroon") == 0)
    return "0x80";

  // Green
  if (strcmp (ColorName,"clGreen") == 0)
    return "0x8000";

  // Olive
  if (strcmp (ColorName,"clOlive") == 0)
    return "0x8080";

  // clNavy
  if (strcmp (ColorName,"clNavy") == 0)
    return "0x800000";

  // clPurple
  if (strcmp (ColorName,"clPurple") == 0)
    return "0x800080";

  // clTeal
  if (strcmp (ColorName,"clTeal") == 0)
    return "0x808000";

  // clGray
  if (strcmp (ColorName,"clGray") == 0)
    return "0x808080";

  // clSilver
  if (strcmp (ColorName,"clSilver") == 0)
    return "0xc0c0c0";

  // Red
  if (strcmp (ColorName,"clRed") == 0)
    return "0xff";

  // clLime
  if (strcmp (ColorName,"clLime") == 0)
    return "0xff00";

  // clYellow
  if (strcmp (ColorName,"clYellow") == 0)
    return "0xffff";

  // clBlue
  if (strcmp (ColorName,"clBlue") == 0)
    return "0xff0000";

  // clFuchsia
  if (strcmp (ColorName,"clFuchsia") == 0)
    return "0xff00ff";

  // clAqua
  if (strcmp (ColorName,"clAqua") == 0)
    return "0xffff00";

  // clLtGray
  if (strcmp (ColorName,"clLtGray") == 0)
    return "0xc0c0c0";

  // clDkGray
  if (strcmp (ColorName,"clDkGray") == 0)
    return "0x808080";

  // clWhite
  if (strcmp (ColorName,"clWhite") == 0)
    return "0xffffff";

  // unknow color name
  return "";
}

//---------------------------------------------------------------------------
// GetFontCharset
//---------------------------------------------------------------------------

char *DFMParser::GetFontCharset (char *Charset)
{
  // ANSI characters
  if (strcmp (Charset,"ANSI_CHARSET") == 0)
    return "FONTENCODING_DEFAULT";

  // Default charset
  if (strcmp (Charset,"DEFAULT_CHARSET") == 0)
    return "FONTENCODING_DEFAULT";

/*
SYMBOL_CHARSET	        2	Standard symbol set.
MAC_CHARSET	        77	Macintosh characters. Not available on NT 3.51.
SHIFTJIS_CHARSET	128	Japanese shift-jis characters.
HANGEUL_CHARSET	        129	Korean characters (Wansung).
JOHAB_CHARSET	        130	Korean characters (Johab). Not available on NT 3.51
GB2312_CHARSET	        134	Simplified Chinese characters (mainland china).
CHINESEBIG5_CHARSET	136	Traditional Chinese characters (taiwanese).
VIETNAMESE_CHARSET	163	Vietnamese characters. Not available on NT 3.51.
THAI_CHARSET	        222	Thai characters. Not available on NT 3.51
OEM_CHARSET	        255	Depends on the codepage of the operating system.
*/

  // Hebrew characters. Not available on NT 3.51
  if (strcmp (Charset,"HEBREW_CHARSET") == 0)
    return "FONTENCODING_ISO_8859_8";

  // Arabic characters. Not available on NT 3.51
  if (strcmp (Charset,"ARABIC_CHARSET") == 0)
    return "FONTENCODING_ISO_8859_6";

  // Baltic characters. Not available on NT 3.51
  if (strcmp (Charset,"BALTIC_CHARSET") == 0)
    return "FONTENCODING_ISO_8859_13";

  // Greek characters. Not available on NT 3.51.
  if (strcmp (Charset,"GREEK_CHARSET") == 0)
    return "FONTENCODING_ISO_8859_7";

  // Turkish characters. Not available on NT 3.51
  if (strcmp (Charset,"TURKISH_CHARSET") == 0)
    return "FONTENCODING_ISO_8859_9";

  // Cyrillic characters. Not available on NT 3.51.
  if (strcmp (Charset,"RUSSIAN_CHARSET") == 0)
    return "FONTENCODING_ISO_8859_5";

  // Includes diacritical marks for eastern european countries. Not available on NT 3.51.
  if (strcmp (Charset,"EASTEUROPE_CHARSET") == 0)
    return "FONTENCODING_ISO_8859_2";

  // otherwise else...
  return "FONTENCODING_DEFAULT";

/// Font character set encoding
/*
enum FXFontEncoding {
  FONTENCODING_DEFAULT      = 0,                        /// Don't care character encoding
  FONTENCODING_ISO_8859_1   = 1,
  FONTENCODING_ISO_8859_2   = 2,
  FONTENCODING_ISO_8859_3   = 3,
  FONTENCODING_ISO_8859_4   = 4,
  FONTENCODING_ISO_8859_5   = 5,                        /// Cyrillic (almost obsolete)
  FONTENCODING_ISO_8859_6   = 6,
  FONTENCODING_ISO_8859_7   = 7,
  FONTENCODING_ISO_8859_8   = 8,
  FONTENCODING_ISO_8859_9   = 9,
  FONTENCODING_ISO_8859_10  = 10,
  FONTENCODING_ISO_8859_11  = 11,
  FONTENCODING_ISO_8859_13  = 13,
  FONTENCODING_ISO_8859_14  = 14,
  FONTENCODING_ISO_8859_15  = 15,
  FONTENCODING_ISO_8859_16  = 16,
  FONTENCODING_KOI8         = 17,
  FONTENCODING_KOI8_R       = 18,                       /// Russian
  FONTENCODING_KOI8_U       = 19,                       /// Ukrainian
  FONTENCODING_KOI8_UNIFIED = 20,

  FONTENCODING_LATIN1      = FONTENCODING_ISO_8859_1,   /// Latin 1 (West European)
  FONTENCODING_LATIN2      = FONTENCODING_ISO_8859_2,   /// Latin 2 (East European)
  FONTENCODING_LATIN3      = FONTENCODING_ISO_8859_3,   /// Latin 3 (South European)
  FONTENCODING_LATIN4      = FONTENCODING_ISO_8859_4,   /// Latin 4 (North European)
  FONTENCODING_LATIN5      = FONTENCODING_ISO_8859_9,   /// Latin 5 (Turkish)
  FONTENCODING_LATIN6      = FONTENCODING_ISO_8859_10,  /// Latin 6 (Nordic)
  FONTENCODING_LATIN7      = FONTENCODING_ISO_8859_13,  /// Latin 7 (Baltic Rim)
  FONTENCODING_LATIN8      = FONTENCODING_ISO_8859_14,  /// Latin 8 (Celtic)
  FONTENCODING_LATIN9      = FONTENCODING_ISO_8859_15,  /// Latin 9 AKA Latin 0
  FONTENCODING_LATIN10     = FONTENCODING_ISO_8859_16,  /// Latin 10

  FONTENCODING_USASCII     = FONTENCODING_ISO_8859_1,   /// Latin 1
  FONTENCODING_WESTEUROPE  = FONTENCODING_ISO_8859_1,   /// Latin 1 (West European)
  FONTENCODING_SOUTHEUROPE = FONTENCODING_ISO_8859_3,   /// Latin 3 (South European)
  FONTENCODING_NORTHEUROPE = FONTENCODING_ISO_8859_4,   /// Latin 4 (North European)
  FONTENCODING_NORDIC      = FONTENCODING_ISO_8859_10,  /// Latin 6 (Nordic)
  FONTENCODING_THAI        = FONTENCODING_ISO_8859_11,  /// Thai
  FONTENCODING_BALTIC      = FONTENCODING_ISO_8859_13,  /// Latin 7 (Baltic Rim)
  FONTENCODING_CELTIC      = FONTENCODING_ISO_8859_14   /// Latin 8 (Celtic)
  };
*/
}

//---------------------------------------------------------------------------
// Get Alignment
//---------------------------------------------------------------------------

char *DFMParser::GetAlignment (char *Alignment)
{
  // taLeftJustify  (default)
  if (Alignment == NULL || strcmp (Alignment,"taLeftJustify") == 0)
    return "JUSTIFY_LEFT";

  // taCenter
  if (strcmp (Alignment,"taCenter") == 0)
    return "JUSTIFY_CENTER_X";

  // taRightJustify
  if (strcmp (Alignment,"taRightJustify") == 0)
    return "JUSTIFY_RIGHT";

  // ?
  return "";
}

//---------------------------------------------------------------------------
// Get Layout
//---------------------------------------------------------------------------

char *DFMParser::GetLayout (char *Layout)
{
  // tlTop (default)
  if (Layout == NULL || strcmp (Layout,"tlTop") == 0)
    return "JUSTIFY_TOP";

  // tlCenter
  if (strcmp (Layout,"tlCenter") == 0)
    return "JUSTIFY_CENTER_Y";

  // tlBottom
  if (strcmp (Layout,"tlBottom") == 0)
    return "JUSTIFY_BOTTOM";

  // ?
  return "";
}

//---------------------------------------------------------------------------
// Get Align
//---------------------------------------------------------------------------

char *DFMParser::GetAlign (char *Align)
{
  // alNone (default)
  if (Align == NULL || strcmp (Align,"alNone") == 0)
    return "LAYOUT_FIX_X | LAYOUT_FIX_Y | LAYOUT_FIX_WIDTH | LAYOUT_FIX_HEIGHT";

  // alTop
  if (strcmp (Align,"alTop") == 0)
    return "LAYOUT_FILL_X | LAYOUT_SIDE_TOP | LAYOUT_FIX_HEIGHT";

  // alBottom
  if (strcmp (Align,"alBottom") == 0)
    return "LAYOUT_FILL_X | LAYOUT_SIDE_BOTTOM | LAYOUT_FIX_HEIGHT";

  // alLeft
  if (strcmp (Align,"alLeft") == 0)
    return "LAYOUT_SIDE_LEFT | LAYOUT_FILL_Y | LAYOUT_FIX_WIDTH";

  // alRight
  if (strcmp (Align,"alRight") == 0)
    return "LAYOUT_SIDE_RIGHT | LAYOUT_FILL_Y | LAYOUT_FIX_WIDTH";

  // alClient
  if (strcmp (Align,"alClient") == 0)
    return "LAYOUT_FILL_X | LAYOUT_FILL_Y";

  // ?
  return "";
}

//---------------------------------------------------------------------------
// Get Cursor
//---------------------------------------------------------------------------

char *DFMParser::GetCursor (char *Cursor)
{
/*
/// Default cursors provided by the application
enum FXDefaultCursor {
  DEF_ARROW_CURSOR,                     /// Arrow cursor
  DEF_RARROW_CURSOR,                    /// Reverse arrow cursor
//  DEF_TEXT_CURSOR,                      /// Text cursor
//  DEF_HSPLIT_CURSOR,                    /// Horizontal split cursor
//  DEF_VSPLIT_CURSOR,                    /// Vertical split cursor
  DEF_XSPLIT_CURSOR,                    /// Cross split cursor
  DEF_SWATCH_CURSOR,                    /// Color swatch drag cursor
  DEF_MOVE_CURSOR,                      /// Move cursor
  DEF_DRAGH_CURSOR,                     /// Resize horizontal edge
  DEF_DRAGV_CURSOR,                     /// Resize vertical edge
  DEF_DRAGTL_CURSOR,                    /// Resize upper-leftcorner
  DEF_DRAGBR_CURSOR=DEF_DRAGTL_CURSOR,  /// Resize bottom-right corner
  DEF_DRAGTR_CURSOR,                    /// Resize upper-right corner
  DEF_DRAGBL_CURSOR=DEF_DRAGTR_CURSOR,  /// Resize bottom-left corner
//  DEF_DNDSTOP_CURSOR,                   /// Drag and drop stop
  DEF_DNDCOPY_CURSOR,                   /// Drag and drop copy
//  DEF_DNDMOVE_CURSOR,                   /// Drag and drop move
  DEF_DNDLINK_CURSOR,                   /// Drag and drop link
//  DEF_CROSSHAIR_CURSOR,                 /// Cross hair cursor
  DEF_CORNERNE_CURSOR,                  /// North-east cursor
  DEF_CORNERNW_CURSOR,                  /// North-west cursor
  DEF_CORNERSE_CURSOR,                  /// South-east cursor
  DEF_CORNERSW_CURSOR,                  /// South-west cursor
  DEF_ROTATE_CURSOR                     /// Rotate cursor
  };
*/
  // crDefault
  if (strcmp (Cursor,"crDefault") == 0)
    return "";

  // crNone
  if (strcmp (Cursor,"crNone") == 0)
    return "";

  // crArrow
  if (strcmp (Cursor,"crArrow") == 0)
    return "DEF_ARROW_CURSOR";

  // crCross
  if (strcmp (Cursor,"crCross") == 0)
    return "DEF_CROSSHAIR_CURSOR";

  // crIBeam
  if (strcmp (Cursor,"crIBeam") == 0)
    return "DEF_TEXT_CURSOR";

  // crSizeAll, crSize
  if (strcmp (Cursor,"crSizeAll") == 0 || strcmp (Cursor,"crSize") == 0)
    return "";

  // crSizeNESW
  if (strcmp (Cursor,"crSizeNESW") == 0)
    return "";

  // crSizeNS
  if (strcmp (Cursor,"crSizeNS") == 0)
    return "";

  // crSizeNWSE
  if (strcmp (Cursor,"crSizeNWSE") == 0)
    return "";

  // crSizeWE
  if (strcmp (Cursor,"crSizeWE") == 0)
    return "";

  // crUpArrow
  if (strcmp (Cursor,"crUpArrow") == 0)
    return "";

  // crHourGlass
  if (strcmp (Cursor,"crHourGlass") == 0)
    return "";

  // crDrag
  if (strcmp (Cursor,"crDrag") == 0)
    return "DEF_DNDMOVE_CURSOR";

  // crNoDrop
  if (strcmp (Cursor,"crNoDrop") == 0)
    return "DEF_DNDSTOP_CURSOR";

  // crHSplit
  if (strcmp (Cursor,"crHSplit") == 0)
    return "DEF_HSPLIT_CURSOR";

  // crVSplit
  if (strcmp (Cursor,"crVSplit") == 0)
    return "DEF_VSPLIT_CURSOR";

  // crMultiDrag
  if (strcmp (Cursor,"crMultiDrag") == 0)
    return "";

  // crSQLWait
  if (strcmp (Cursor,"crSQLWait") == 0)
    return "";

  // crNo
  if (strcmp (Cursor,"crNo") == 0)
    return "";

  // crAppStart
  if (strcmp (Cursor,"crAppStart") == 0)
    return "";

  // crHelp
  if (strcmp (Cursor,"crHelp") == 0)
    return "";

  // crHandPoint
  if (strcmp (Cursor,"crHandPoint") == 0)
    return "";

  // ?
  return "";
}

//---------------------------------------------------------------------------
// Save Icon/Glyph file
//---------------------------------------------------------------------------

void DFMParser::SaveIconGlyphFile (char *Name,char *Data)
{
  FILE *file = fopen (Name,"wb");
  for (unsigned int i = 0; i < strlen (Data); i += 2)

    {

      unsigned char c = 0;


      if (Data [i] >= 'A')

        c = (unsigned char)(Data [i] - 55);

      else

        c = (unsigned char)(Data [i] - 48);

      c <<= 4;

      if (Data [i + 1] >= 'A')

        c += (unsigned char)(Data [i + 1] - 55);

      else

        c += (unsigned char)(Data [i + 1] - 48);

      fwrite (&c,1,1,file);

    }

  fclose (file);

}



