#ifndef __DFMPARSER_H__
#define __DFMPARSER_H__

//---------------------------------------------------------------------------
// generate class type
//---------------------------------------------------------------------------

enum ClassGenerateType
{
  DIALOG_BOX,
  MAIN_WINDOW
};

//---------------------------------------------------------------------------
// DFMParser class
//---------------------------------------------------------------------------

class DFMParser
{
public:
  DFMParser (char *FileName);
  virtual ~DFMParser ();
  void ParseFile ();
  void LoadFile (char *FileName);
  char *GetTextBuf () { return TextBuf; }
  int GetTextSize () { return TextSize; }
  ClassGenerateType GetClassGenerateType () { return ClassGenerate; }
  void SetClassGenerateType (ClassGenerateType CGT) { ClassGenerate = CGT; }
  void SetSaveICO (bool Save) { SaveICO = Save; }
  bool GetSaveICO () { return SaveICO; }
  void SetSaveGlyph (bool Save) { SaveGlyph = Save; }
  bool GetSaveGlyph () { return SaveGlyph; }
  void SetSavePicture (bool Save) { SavePicture = Save; }
  bool GetSavePicturee () { return SavePicture; }
  void SetGenerateMain (bool Generate) { GenerateMain = Generate; }
  bool GetGenerateMain () { return GenerateMain; }
protected:
  ClassGenerateType ClassGenerate;
  bool SaveICO;
  bool SaveGlyph;
  bool SavePicture;
  bool GenerateMain;
  StackElement *Stack;
  char *TextBuf;
  int TextSize;
  int Pos;
  int ReadDigit (bool &Error);
  char *ReadString (bool &Error);
  char *ReadQuotation (bool &Error);
  char *ReadIcon (bool &Error);
  void SkipChar (char SkipChar);
  void GetClass (char *ClassType);
  void ReadStringProperty (char *Name,char *Property,char *&Value);
  void ReadDigitProperty (char *Name,char *Property,int &Value);
  void ReadQuotationProperty (char *Name,char *Property,char *&Value);
  void ReadSetProperty (char *Name,char *Property,StackElement *&Value);
protected:
  char *GetAlignment (char *Alignment);
  char *GetLayout (char *Layout);
  char *GetAlign (char *Align);
  char *GetColor (char *ColorName);
  char *GetFontCharset (char *Charset);
  char *GetCursor (char *Cursor);
  void SaveIconGlyphFile (char *Name,char *Data);
protected:
  // VCL Class
  void TFormClass ();
  void TLabelClass ();
  void TStaticTextClass ();
  void TPanelClass ();
  void TBevelClass ();
  void TButtonClass ();
  void TMainMenuClass ();
  void TMenuItemClass ();
  void TGroupBoxClass ();
  void TRadioGroupClass ();
  void TComboBoxClass ();
  void TListBoxClass ();
  void TEditClass ();
  void TMemoClass ();
  void TBitBtnClass ();
  void TSpeedButtonClass ();
  void TImageClass ();
  void TCheckBoxClass ();
protected:
  FILE *cppfile;
  FILE *hfile;
protected:
  int MenuDepth;
  int IconCount;
  int GlyphCount;
  StackElement *EventsFunction;
  StackElement *MessageID;
  StackElement *MessageType;
};

//---------------------------------------------------------------------------
// DFMParser class exception
//---------------------------------------------------------------------------

enum DFMParser_Exception
{
  DFM_PARSER_ERROR_OPEN_FILE,    // open file error
  DFM_PARSER_ERROR_READ_FILE,    // read file error
  DFM_PARSER_ERROR_WRITE_FILE    // save file error
};

#endif // __DFMPARSER_H__
