//---------------------------------------------------------------------------
// TComboBox Class
//---------------------------------------------------------------------------

/*
Supported properties:

  Left
  Top
  Width
  Height
  Text
  Color
  Hint
  Font.Color
  Font.Charset
  Font.Height
  Font.Name
  Font.Style
  Items.Strings
  Cursor
  Enabled
*/

void DFMParser::TComboBoxClass ()
{
  // attributes
  int  Left;
  int  Top;
  int  Width;
  int  Height;
  char *Color = NULL;
  char *Cursor = NULL;
  char *Text = NULL;
  char *FontCharset = NULL;
  int FontHeight = 0;
  char *FontName = NULL;
  StackElement *FontStyle = NULL;
  char *FontColor = NULL;
  StackElement *ItemsStrings = NULL;
  char *Hint = NULL;
  char *Enabled = NULL;

  // read words...
  bool Error;
  char *Name = ReadString (Error);
  while (strcmp (Name,"object") != 0 && strcmp (Name,"end") != 0)
    {
      // Left
      if (strcmp (Name,"Left") == 0)
        {
          SkipChar ('=');
          Left = ReadDigit (Error);
        }
      else

      // Top
      if (strcmp (Name,"Top") == 0)
        {
          SkipChar ('=');
          Top = ReadDigit (Error);
        }
      else

      // Height
      if (strcmp (Name,"Height") == 0)
        {
          SkipChar ('=');
          Height = ReadDigit (Error);
        }
      else

      // Width
      if (strcmp (Name,"Width") == 0)
        {
          SkipChar ('=');
          Width = ReadDigit (Error);
        }
      else

      // Color
      if (strcmp (Name,"Color") == 0)
        {
          SkipChar ('=');
          Color = ReadString (Error);
        }
      else

      // Cursor
      if (strcmp (Name,"Cursor") == 0)
        {
          SkipChar ('=');
          Cursor = ReadString (Error);
        }
      else

      // Text
      if (strcmp (Name,"Text") == 0)
        {
          SkipChar ('=');
          Text = ReadQuotation (Error);
        }
      else

      // Font Charset
      if (strcmp (Name,"Font.Charset") == 0)
        {
          SkipChar ('=');
          FontCharset = ReadString (Error);
        }
      else

      // Font Color
      if (strcmp (Name,"Font.Color") == 0)
        {
          SkipChar ('=');
          FontColor = ReadString (Error);
        }
      else

      // Font Height
      if (strcmp (Name,"Font.Height") == 0)
        {
          SkipChar ('=');
          FontHeight = ReadDigit (Error);
        }
      else

      // Font Name
      if (strcmp (Name,"Font.Name") == 0)
        {
          SkipChar ('=');
          FontName = ReadQuotation (Error);
        }

      // Font Style
      if (strcmp (Name,"Font.Style") == 0)
        {
          SkipChar ('=');
          SkipChar ('[');
          char *Style = NULL;
          while (TextBuf [Pos] != ']')
            {
              Style = ReadString (Error);
              Push (FontStyle,Style);
              delete [] Style;
            }
        }
      else

      // Items.Strings
      if (strcmp (Name,"Items.Strings") == 0)
        {
          SkipChar ('=');
          SkipChar ('(');
          char *ItemString = NULL;
          while (TextBuf [Pos] != ')')
            {
              ItemString = ReadQuotation (Error);
              Push (ItemsStrings,ItemString);
              SkipChar ('\'');
              delete [] ItemString;
            }
        }
      else

      // Hint
      if (strcmp (Name,"Hint") == 0)
        {
          SkipChar ('=');
          Hint = ReadQuotation (Error);
        }
      else

      // Enabled
      if (strcmp (Name,"Enabled") == 0)
        {
          SkipChar ('=');
          Enabled = ReadString (Error);
        }

      // read new word ...
      delete [] Name;
      Name = ReadString (Error);
    }
  delete [] Name;

  // count ItemIndex
  int ItemsCount = 0;
  StackElement *TmpStack = ItemsStrings;
  while (TmpStack != NULL)
    {
      TmpStack = TmpStack -> Next;
      ItemsCount++;
    }

  // number of visual items
  int ItemsVisual = ItemsCount;
  if (ItemsVisual > 8)
    ItemsVisual = 8;
  if (ItemsVisual == 0)
    ItemsVisual = 1;

  // *.h file
  fprintf (hfile,"  FXComboBox *%s;\n",Stack -> Name);

  // *.cpp file
  fprintf (cppfile,"  %s = new FXComboBox (%s,%i,%i,NULL,0,",Stack -> Name,Stack -> Next -> Name,ItemsCount,ItemsVisual);
  fprintf (cppfile,"LAYOUT_FIX_X | LAYOUT_FIX_Y | LAYOUT_FIX_WIDTH | LAYOUT_FIX_HEIGHT | FRAME_SUNKEN | FRAME_THICK,");
  fprintf (cppfile,"%i,%i,%i,%i,0,0,0,0);\n",Left,Top,Width,Height);

  // Text
  if (Text != NULL && strlen (Text) > 0)
    {
      fprintf (cppfile,"  %s -> setText (\"%s\");\n",Stack -> Name,Text);
    }

  // Hint
  if (Hint != NULL && strlen (Hint) > 0)
    {
      fprintf (cppfile,"  %s -> setTipText (\"%s\");\n",Stack -> Name,Hint);
    }

  // reverse Items.Strings stack
  Reverse (ItemsStrings);
/*
  StackElement *ReverseItemsStrings = NULL;
  while (!Empty (ItemsStrings))
    {
      Push (ReverseItemsStrings,ItemsStrings -> Name);
      Pop (ItemsStrings);
    }
*/
  // ItemsStrings
//  ItemsStrings = ReverseItemsStrings;
  while (!Empty (ItemsStrings))
    {
      // *.cpp file
      fprintf (cppfile,"  %s -> appendItem (\"%s\");\n",Stack -> Name,ItemsStrings -> Name);

      // pop the Item
      Pop (ItemsStrings);
    }

  // Color
  if (Color != NULL && strlen (GetColor (Color)) > 0)
    {
      fprintf (cppfile,"  %s -> setBackColor (%s);\n",Stack -> Name,GetColor (Color));
    }

  // Cursor
  if (Cursor != NULL && strlen (GetCursor (Cursor)) > 0)
    {
      fprintf (cppfile,"  %s -> setDefaultCursor(getApp () -> getDefaultCursor(%s));\n",Stack -> Name,GetCursor (Cursor));
    }
    
  // Font
  if (FontHeight != 0 && FontName != NULL)
    {
      fprintf (cppfile,"  %s -> setFont (new FXFont (getApp(),\"%s\",%i",Stack -> Name,FontName,(int)((-FontHeight * 72) / 96));

      bool Bold = false;
      bool Italic = false;
      while (!Empty (FontStyle))
        {
          if (strcmp (FontStyle -> Name,"fsBold") == 0)
            Bold = true;
          else
          if (strcmp (FontStyle -> Name,"fsItalic") == 0)
            Italic = true;
          Pop (FontStyle);
        }

      // Bold
      if (Bold == true)
        fprintf (cppfile,",FONTWEIGHT_BOLD");
      else
        fprintf (cppfile,",FONTWEIGHT_NORMAL");

      // Italic
      if (Italic == true)
        fprintf (cppfile,",FONTSLANT_ITALIC");
      else
        fprintf (cppfile,",FONTSLANT_REGULAR");

      // Charset
      if (FontCharset != NULL)
        fprintf (cppfile,",%s));\n",GetFontCharset (FontCharset));
      else
        fprintf (cppfile,"));\n");
    }

  // Font Color
  if (FontColor != NULL && strlen (GetColor (FontColor)) > 0)
    {
      fprintf (cppfile,"  %s -> setTextColor (%s);\n",Stack -> Name,GetColor (FontColor));
    }

  // Enabled
  if (Enabled != NULL && strcmp (Enabled,"False") == 0)
    {
      fprintf (cppfile,"  %s -> disable ();\n",Stack -> Name);
    }

  // one empty line
  fprintf (cppfile,"\n");

  // clear
  delete [] Color;
  delete [] Text;
  delete [] Cursor;
  delete [] FontCharset;
  delete [] FontStyle;
  delete [] FontName;
  delete [] FontColor;
  delete [] Hint;
  delete [] Enabled;
}
