//---------------------------------------------------------------------------
// TEdit Class
//---------------------------------------------------------------------------

void DFMParser::TEditClass ()
{
  // attributes
  int  Left;
  int  Top;
  int  Width;
  int  Height;
  char *Text = NULL;
  char *Hint = NULL;
  char *Enabled = NULL;
  char *FontColor = NULL;
  char *FontCharset = NULL;
  int FontHeight = 0;
  char *FontName = NULL;
  StackElement *FontStyle = NULL;
  char *Cursor = NULL;
  char *BorderStyle = NULL;
  char *Color = NULL; 

  // read words...
  bool Error;
  char *Name = ReadString (Error);
  while (strcmp (Name,"object") != 0 && strcmp (Name,"end") != 0)
    {
      // Left
      if (strcmp (Name,"Left") == 0)
        {
          SkipChar ('=');
          Left = ReadDigit (Error);
        }
      else

      // Top
      if (strcmp (Name,"Top") == 0)
        {
          SkipChar ('=');
          Top = ReadDigit (Error);
        }
      else

      // Height
      if (strcmp (Name,"Height") == 0)
        {
          SkipChar ('=');
          Height = ReadDigit (Error);
        }
      else

      // Width
      if (strcmp (Name,"Width") == 0)
        {
          SkipChar ('=');
          Width = ReadDigit (Error);
        }
      else

      // Text
      if (strcmp (Name,"Text") == 0)
        {
          SkipChar ('=');
          Text = ReadQuotation (Error);
        }
      else

      // Hint
      if (strcmp (Name,"Hint") == 0)
        {
          SkipChar ('=');
          Hint = ReadQuotation (Error);
        }
      else

      // Enabled
      if (strcmp (Name,"Enabled") == 0)
        {
          SkipChar ('=');
          Enabled = ReadString (Error);
        }
      else

      // Font Charset
      if (strcmp (Name,"Font.Charset") == 0)
        {
          SkipChar ('=');
          FontCharset = ReadString (Error);
        }
      else

      // Font Height
      if (strcmp (Name,"Font.Height") == 0)
        {
          SkipChar ('=');
          FontHeight = ReadDigit (Error);
        }
      else

      // Font Name
      if (strcmp (Name,"Font.Name") == 0)
        {
          SkipChar ('=');
          FontName = ReadQuotation (Error);
        }
      else

      // Font Color
      if (strcmp (Name,"Font.Color") == 0)
        {
          SkipChar ('=');
          FontColor = ReadString (Error);
        }

      // Font Style
      if (strcmp (Name,"Font.Style") == 0)
        {
          SkipChar ('=');
          SkipChar ('[');
          char *Style = NULL;
          while (TextBuf [Pos] != ']')
            {
              Style = ReadString (Error);
              Push (FontStyle,Style);
              delete [] Style;
            }
        }
      else

      // Cursor
      if (strcmp (Name,"Cursor") == 0)
        {
          SkipChar ('=');
          Cursor = ReadString (Error);
        }
      else

      // BorderStyle
      if (strcmp (Name,"BorderStyle") == 0)
        {
          SkipChar ('=');
          BorderStyle = ReadString (Error);
        }
      else

      // Color
      if (strcmp (Name,"Color") == 0)
        {
          SkipChar ('=');
          Color = ReadString (Error);
        }

      // read new word ...
      delete [] Name;
      Name = ReadString (Error);
    }
  delete [] Name;

  // *.h file
  fprintf (hfile,"  FXTextField *%s;\n",Stack -> Name);

  // *.cpp file
  fprintf (cppfile,"  %s = new FXTextField (%s,0,NULL,0,",Stack -> Name,Stack -> Next -> Name);

  // BorderStyle
  if (BorderStyle == NULL || strcmp (BorderStyle,"bsSingle") == 0)
    {
      fprintf (cppfile,"FRAME_NORMAL | ");
    }

  fprintf (cppfile,"JUSTIFY_LEFT | LAYOUT_FIX_X | LAYOUT_FIX_Y | LAYOUT_FIX_WIDTH | LAYOUT_FIX_HEIGHT,");
  fprintf (cppfile,"%i,%i,%i,%i,0,0,0,0);\n",Left,Top,Width,Height);

  // Text
  if (Text != NULL)
    {
      fprintf (cppfile,"  %s -> setText (\"%s\");\n",Stack -> Name,Text);
    }

  // Hint
  if (Hint != NULL)
    {
      fprintf (cppfile,"  %s -> setTipText (\"%s\");\n",Stack -> Name,Hint);
    }

  // Enabled
  if (Enabled != NULL && strcmp (Enabled,"False") == 0)
    {
      fprintf (cppfile,"  %s -> disable ();\n",Stack -> Name);
    }

  // Font Color

  if (FontColor != NULL && strlen (GetColor (FontColor)) > 0)
    {
      fprintf (cppfile,"  %s -> setTextColor (%s);\n",Stack -> Name,GetColor (FontColor));
    }

  // Font
  if (FontHeight != 0 && FontName != NULL)
    {
      fprintf (cppfile,"  %s -> setFont (new FXFont (getApp(),\"%s\",%i",Stack -> Name,FontName,(int)((-FontHeight * 72) / 96));

      bool Bold = false;
      bool Italic = false;
      while (!Empty (FontStyle))
        {
          if (strcmp (FontStyle -> Name,"fsBold") == 0)
            Bold = true;
          else
          if (strcmp (FontStyle -> Name,"fsItalic") == 0)
            Italic = true;
          Pop (FontStyle);
        }
      if (Bold == true)
        fprintf (cppfile,",FONTWEIGHT_BOLD");
      else
        fprintf (cppfile,",FONTWEIGHT_NORMAL");

      if (Italic == true)
        fprintf (cppfile,",FONTSLANT_ITALIC");
      else
        fprintf (cppfile,",FONTSLANT_REGULAR");

      if (FontCharset != NULL)
        fprintf (cppfile,",%s));\n",GetFontCharset (FontCharset));
      else
        fprintf (cppfile,"));\n");
    }

  // Cursor
  if (Cursor != NULL && strlen (GetCursor (Cursor)) > 0)
    {
      fprintf (cppfile,"  %s -> setDefaultCursor(getApp () -> getDefaultCursor(%s));\n",Stack -> Name,GetCursor (Cursor));
    }
    
  // Color
  if (Color != NULL && strlen (GetColor (Color)) > 0)
    {
      fprintf (cppfile,"  %s -> setBackColor (%s);\n",Stack -> Name,GetColor (Color));
    }

  // one empty line
  fprintf (cppfile,"\n");

  // clear
  delete [] Text;
  delete [] FontColor;
  delete [] Hint;
  delete [] FontCharset;
  delete [] FontName;
  delete [] Enabled;
  delete [] Cursor;
  delete [] BorderStyle;
  delete [] Color;
}
