//---------------------------------------------------------------------------
// TGroupBoxObject Class
//---------------------------------------------------------------------------

/*
Supported properties:

  Left
  Top
  Width
  Height
  Color
  Align
  Cursor
  Caption
  Font.Charset
  Font.Height
  Font.Name
  Font.Style
  Font.Color
  Enabled

Supported events
  OnClick
  OnMouseDown
  OnMouseUp
*/

void DFMParser::TGroupBoxClass ()
{
  // attributes
  int Left;
  int Top;
  int Width;
  int Height;
  char *Color = NULL;
  char *Align = NULL;
  char *Cursor = NULL;
  char *Caption = NULL;
  char *FontCharset = NULL;
  int FontHeight = 0;
  char *FontName = NULL;
  StackElement *FontStyle = NULL;
  char *FontColor = NULL;
  char *Enabled = NULL;

  // events
  char *OnClick = NULL;
  char *OnMouseDown = NULL;
  char *OnMouseUp = NULL;

  // read words...
  bool Error;
  char *Name = ReadString (Error);
  while (strcmp (Name,"object") != 0 && strcmp (Name,"end") != 0)
    {
      // Left
      ReadDigitProperty (Name,"Left",Left);

      // Top
      ReadDigitProperty (Name,"Top",Top);

      // Height
      ReadDigitProperty (Name,"Height",Height);

      // Width
      ReadDigitProperty (Name,"Width",Width);

      // Color
      ReadStringProperty (Name,"Color",Color);

      // Align
      ReadStringProperty (Name,"Align",Align);

      // Cursor
      ReadStringProperty (Name,"Cursor",Cursor);

      // Caption
      ReadQuotationProperty (Name,"Caption",Caption);

      // Font Charset
      ReadStringProperty (Name,"Font.Charset",FontCharset);

      // Font Color
      ReadStringProperty (Name,"Font.Color",FontColor);

      // Font Height
      ReadDigitProperty (Name,"Font.Height",FontHeight);

      // Font Name
      ReadQuotationProperty (Name,"Font.Name",FontName);

      // Font Style
      ReadSetProperty (Name,"Font.Style",FontStyle);

      // Enabled
      ReadStringProperty (Name,"Enabled",Enabled);

      // OnClick
      ReadStringProperty (Name,"OnClick",OnClick);

      // OnMouseDown
      ReadStringProperty (Name,"OnMouseDown",OnMouseDown);

      // OnMouseUp
      ReadStringProperty (Name,"OnMouseUp",OnMouseUp);

      // read new word ...
      delete [] Name;
      Name = ReadString (Error);
    }
  delete [] Name;

  // *.h file
  fprintf (hfile,"  FXGroupBox *%s;\n",Stack -> Name);

  // *.cpp file
  fprintf (cppfile,"  %s = new FXGroupBox (%s,\"",Stack -> Name,Stack -> Next -> Name);

  // Caption
  if (Caption != NULL && strlen (Caption) > 0)
    {
      fprintf (cppfile,"%s",Caption);
    }

  // Align and other...
  fprintf (cppfile,"\",%s | FRAME_THICK,%i,%i,%i,%i,0,0,0,0);\n",GetAlign (Align),Left,Top,Width,Height);

  // Color
  if (Color != NULL && strlen (GetColor (Color)) > 0)
    {
      fprintf (cppfile,"  %s -> setBackColor (%s);\n",Stack -> Name,GetColor (Color));
    }

  // Cursor
  if (Cursor != NULL && strlen (GetCursor (Cursor)) > 0)
    {
      fprintf (cppfile,"  %s -> setDefaultCursor(getApp () -> getDefaultCursor(%s));\n",Stack -> Name,GetCursor (Cursor));
    }

  // Font
  if (FontHeight != 0 && FontName != NULL)
    {
      fprintf (cppfile,"  %s -> setFont (new FXFont (getApp(),\"%s\",%i",Stack -> Name,FontName,(int)((-FontHeight * 72) / 96));

      bool Bold = false;
      bool Italic = false;
      while (!Empty (FontStyle))
        {
          if (strcmp (FontStyle -> Name,"fsBold") == 0)
            Bold = true;
          else
          if (strcmp (FontStyle -> Name,"fsItalic") == 0)
            Italic = true;
          Pop (FontStyle);
        }

      // Bold
      if (Bold == true)
        fprintf (cppfile,",FONTWEIGHT_BOLD");
      else
        fprintf (cppfile,",FONTWEIGHT_NORMAL");

      // Italic
      if (Italic == true)
        fprintf (cppfile,",FONTSLANT_ITALIC");
      else
        fprintf (cppfile,",FONTSLANT_REGULAR");

      // Charset
      if (FontCharset != NULL)
        fprintf (cppfile,",%s));\n",GetFontCharset (FontCharset));
      else
        fprintf (cppfile,"));\n");
    }

  // Font Color
  if (FontColor != NULL && strlen (GetColor (FontColor)) > 0)
    {
      fprintf (cppfile,"  %s -> setTextColor (%s);\n",Stack -> Name,GetColor (FontColor));
    }

  // Enabled
  if (Enabled != NULL && strcmp (Enabled,"False") == 0)
    {
      fprintf (cppfile,"  %s -> disable ();\n",Stack -> Name);
    }

  // Event - OnClick
  if (OnClick != NULL)
    {
      // EventFunction
      Push (EventsFunction,OnClick);

      // Message Type
      Push (MessageType,"SEL_LEFTBUTTONPRESS");

      // MessageID
      char MsgID [512];
      sprintf (MsgID,"ID_%s",Stack -> Name);
      Push (MessageID,MsgID);
    }

  // Event - OnMouseDown, it's simulated by SEL_LEFTBUTTONPRESS,
  // SEL_MIDDLEBUTTONPRESS and SEL_RIGHTBUTTONPRESS

  if (OnMouseDown != NULL)
    {
      // Event functions
      Push (EventsFunction,OnMouseDown);
      Push (EventsFunction,OnMouseDown);
      Push (EventsFunction,OnMouseDown);

      // Message types
      Push (MessageType,"SEL_LEFTBUTTONPRESS");
      Push (MessageType,"SEL_MIDDLEBUTTONPRESS");
      Push (MessageType,"SEL_RIGHTBUTTONPRESS");

      // MessageIDs
      char MsgID [512];
      sprintf (MsgID,"ID_%s",Stack -> Name);
      Push (MessageID,MsgID);
      Push (MessageID,MsgID);
      Push (MessageID,MsgID);
    }

  // Event - OnMouseUp, it's simulated by SEL_LEFTBUTTONRELEASE,
  // SEL_MIDDLEBUTTONRELEASE and SEL_RIGHTBUTTONRELEASE

  if (OnMouseUp != NULL)

    {
      // Event functions
      Push (EventsFunction,OnMouseUp);
      Push (EventsFunction,OnMouseUp);
      Push (EventsFunction,OnMouseUp);

      // Message types
      Push (MessageType,"SEL_LEFTBUTTONRELEASE");
      Push (MessageType,"SEL_MIDDLEBUTTONRELEASE");
      Push (MessageType,"SEL_RIGHTBUTTONRELEASE");

      // MessageIDs
      char MsgID [512];
      sprintf (MsgID,"ID_%s",Stack -> Name);
      Push (MessageID,MsgID);
      Push (MessageID,MsgID);
      Push (MessageID,MsgID);
    }

  // any Event
  if (OnClick != NULL || OnMouseDown != NULL || OnMouseUp != NULL)
    {
      // MessageID
      char MsgID [512];
      sprintf (MsgID,"ID_%s",Stack -> Name);

      // select Target and Selector
      fprintf (cppfile,"  %s -> setTarget (this);\n",Stack -> Name);
      fprintf (cppfile,"  %s -> setSelector (%s);\n",Stack -> Name,MsgID);
    }

  // one empty line
  fprintf (cppfile,"\n");

  // clear
  delete [] Color;
  delete [] Align;
  delete [] Cursor;
  delete [] Caption;
  delete [] FontCharset;
  delete [] FontStyle;
  delete [] FontName;
  delete [] FontColor;
  delete [] Enabled;
  delete [] OnClick;
  delete [] OnMouseDown;
  delete [] OnMouseUp;
}
