//---------------------------------------------------------------------------
// TSpeedButton Class
//---------------------------------------------------------------------------

/*
Supported properties:

  Left
  Top
  Width
  Height
  Caption
  Hint
  Enabled
  Font.Color
  Font.Charset
  Font.Height
  Font.Name
  Font.Style
  Cursor
  GlyphData
  Layout
  Flat
*/

void DFMParser::TSpeedButtonClass ()
{
  // properties
  int  Left;
  int  Top;
  int  Width;
  int  Height;
  char *Caption = NULL;
  char *Hint = NULL;
  char *Enabled = NULL;
  char *FontColor = NULL;
  char *FontCharset = NULL;
  int FontHeight = 0;
  char *FontName = NULL;
  StackElement *FontStyle = NULL;
  char *Cursor = NULL;
  char *GlyphData = NULL;
  char *Layout = NULL;
  char *Flat = NULL;

  // Events
  char *OnClick = NULL;
  char *OnKeyDown = NULL;
  char *OnKeyUp = NULL;

  // read words...
  bool Error;
  char *Name = ReadString (Error);
  while (strcmp (Name,"object") != 0 && strcmp (Name,"end") != 0)
    {
      // Left
      if (strcmp (Name,"Left") == 0)
        {
          SkipChar ('=');
          Left = ReadDigit (Error);
        }
      else

      // Top
      if (strcmp (Name,"Top") == 0)
        {
          SkipChar ('=');
          Top = ReadDigit (Error);
        }
      else

      // Height
      if (strcmp (Name,"Height") == 0)
        {
          SkipChar ('=');
          Height = ReadDigit (Error);
        }
      else

      // Width
      if (strcmp (Name,"Width") == 0)
        {
          SkipChar ('=');
          Width = ReadDigit (Error);
        }
      else

      // Caption
      if (strcmp (Name,"Caption") == 0)
        {
          SkipChar ('=');
          Caption = ReadQuotation (Error);
        }
      else

      // Hint
      if (strcmp (Name,"Hint") == 0)
        {
          SkipChar ('=');
          Hint = ReadQuotation (Error);
        }
      else

      // Enabled
      if (strcmp (Name,"Enabled") == 0)
        {
          SkipChar ('=');
          Enabled = ReadString (Error);
        }
      else

      // Font Charset
      if (strcmp (Name,"Font.Charset") == 0)
        {
          SkipChar ('=');
          FontCharset = ReadString (Error);
        }
      else

      // Font Height
      if (strcmp (Name,"Font.Height") == 0)
        {
          SkipChar ('=');
          FontHeight = ReadDigit (Error);
        }
      else

      // Font Name
      if (strcmp (Name,"Font.Name") == 0)
        {
          SkipChar ('=');
          FontName = ReadQuotation (Error);
        }
      else

      // Font Color
      if (strcmp (Name,"Font.Color") == 0)
        {
          SkipChar ('=');
          FontColor = ReadString (Error);
        }
      else

      // Font Style
      if (strcmp (Name,"Font.Style") == 0)
        {
          SkipChar ('=');
          SkipChar ('[');
          char *Style = NULL;
          while (TextBuf [Pos] != ']')
            {
              Style = ReadString (Error);
              Push (FontStyle,Style);
              delete [] Style;
            }
        }
      else

      // Cursor
      if (strcmp (Name,"Cursor") == 0)
        {
          SkipChar ('=');
          Cursor = ReadString (Error);
        }
      else

      // Glyph.Data
      if (strcmp (Name,"Glyph.Data") == 0)
        {
          SkipChar ('=');
          GlyphData = ReadIcon (Error);
        }
      else

      // OnClick
      if (strcmp (Name,"OnClick") == 0)
        {
          SkipChar ('=');
          OnClick = ReadString (Error);
        }
      else

      // OnKeyDown
      if (strcmp (Name,"OnKeyDown") == 0)
        {
          SkipChar ('=');
          OnKeyDown = ReadString (Error);
        }
      else

      // OnKeyUp
      if (strcmp (Name,"OnKeyUp") == 0)
        {
          SkipChar ('=');
          OnKeyUp = ReadString (Error);
        }
      else

      // Flat
      if (strcmp (Name,"Flat") == 0)
        {
          SkipChar ('=');
          Flat = ReadString (Error);
        }
      else

      // Layout
      if (strcmp (Name,"Layout") == 0)
        {
          SkipChar ('=');
          Layout = ReadString (Error);
        }

      // read new word ...
      delete [] Name;
      Name = ReadString (Error);
    }
  delete [] Name;

  // *.h file
  fprintf (hfile,"  FXButton *%s;\n",Stack -> Name);

  // Glyph
  if (GlyphData != NULL)

    {

      // next Glyph file...

      GlyphCount++;

      char name [30];


      // write data

      sprintf (name,"glyph%i",GlyphCount);

      fprintf (cppfile,"  const unsigned char %s []={\n",name);

      fprintf (cppfile,"  ");

      int x = 0;


      // first 4 bytes (8 chars) is added (who no why?), next bytes creates a BMP file

      for (unsigned int i = 8; i < strlen (GlyphData); i += 2)

        {

          fprintf (cppfile,"0x%c%c",GlyphData [i],GlyphData [i+1]);

          if (i < strlen (GlyphData) - 2)

            fprintf (cppfile,",");

          else

            fprintf (cppfile,"};\n");

          if (++x == 16)

            {

              fprintf (cppfile,"\n  ");

              x = 0;

            }

        }

      fprintf (cppfile,"\n");


      // save the glyph file

      if (SaveGlyph == true)

        {

          strcat (name,".bmp");

          SaveIconGlyphFile (name,&GlyphData[8]);

        }

    }

  // *.cpp file
  fprintf (cppfile,"  %s = new FXButton (%s,\"",Stack -> Name,Stack -> Next -> Name);

  // Caption
  if (Caption != NULL)
    {
      fprintf (cppfile,"%s",Caption);
    }

  // Hint
  if (Hint != NULL)
    {
      fprintf (cppfile,"\\t%s",Hint);
    }

  // Glyph
  if (GlyphData != NULL)
    {
      char name [30];
      sprintf (name,"glyph%i",GlyphCount);

      fprintf (cppfile,"\",new FXBMPIcon(getApp(),%s,0,IMAGE_ALPHAGUESS)",name);
    }
  else
    fprintf (cppfile,"\",NULL");

  // Event - OnClick, OnKeyDown or OnKeyUp
  if (OnClick != NULL || OnKeyDown != NULL || OnKeyUp != NULL)
    {
      fprintf (cppfile,",this,ID_%s",Stack -> Name,OnClick);
    }
  else
    fprintf (cppfile,",NULL,0");

  // Layout

  if (Layout != NULL)
    {
      if (strcmp (Layout,"blGlyphLeft") == 0)
        fprintf (cppfile,",ICON_BEFORE_TEXT|");
      else
      if (strcmp (Layout,"blGlyphRight") == 0)
        fprintf (cppfile,",ICON_AFTER_TEXT|");
      else
      if (strcmp (Layout,"blGlyphTop") == 0)
        fprintf (cppfile,",ICON_ABOVE_TEXT|");
      else
      if (strcmp (Layout,"blGlyphBottom") == 0)
        fprintf (cppfile,",ICON_BELOW_TEXT|");
    }
  else
    fprintf (cppfile,",ICON_BEFORE_TEXT|");

  // Flat
  if (Flat != NULL && strcmp (Flat,"True") == 0)
    fprintf (cppfile,"BUTTON_TOOLBAR|FRAME_RAISED|");
  else
    fprintf (cppfile,"FRAME_NORMAL|");

  fprintf (cppfile,"LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,");
  fprintf (cppfile,"%i,%i,%i,%i,0,0,0,0);\n",Left,Top,Width,Height);

  // Enabled
  if (Enabled != NULL && strcmp (Enabled,"False") == 0)
    {
      fprintf (cppfile,"  %s -> disable ();\n",Stack -> Name);
    }

  // Font Color

  if (FontColor != NULL && strlen (GetColor (FontColor)) > 0)
    {
      fprintf (cppfile,"  %s -> setTextColor (%s);\n",Stack -> Name,GetColor (FontColor));
    }

  // Font
  if (FontHeight != 0 && FontName != NULL)
    {
      fprintf (cppfile,"  %s -> setFont (new FXFont (getApp(),\"%s\",%i",Stack -> Name,FontName,(int)((-FontHeight * 72) / 96));

      bool Bold = false;
      bool Italic = false;
      while (!Empty (FontStyle))
        {
          if (strcmp (FontStyle -> Name,"fsBold") == 0)
            Bold = true;
          else
          if (strcmp (FontStyle -> Name,"fsItalic") == 0)
            Italic = true;
          Pop (FontStyle);
        }

      // Bold
      if (Bold == true)
        fprintf (cppfile,",FONTWEIGHT_BOLD");
      else
        fprintf (cppfile,",FONTWEIGHT_NORMAL");

      // Italic
      if (Italic == true)
        fprintf (cppfile,",FONTSLANT_ITALIC");
      else
        fprintf (cppfile,",FONTSLANT_REGULAR");

      // FontCharset
      if (FontCharset != NULL)
        fprintf (cppfile,",%s));\n",GetFontCharset (FontCharset));
      else
        fprintf (cppfile,"));\n");
    }

  // Cursor
  if (Cursor != NULL && strlen (GetCursor (Cursor)) > 0)
    {
      fprintf (cppfile,"  %s -> setDefaultCursor(getApp () -> getDefaultCursor(%s));\n",Stack -> Name,GetCursor (Cursor));
    }

  // one empty line
  fprintf (cppfile,"\n");

  // Event - OnClick
  if (OnClick != NULL)
    {
      // EventFunction
      Push (EventsFunction,OnClick);

      // Message Type
      Push (MessageType,"SEL_COMMAND");

      // MessageID
      char MsgID [512];
      sprintf (MsgID,"ID_%s",Stack -> Name);
      Push (MessageID,MsgID);
    }

  // Event - OnKeyDown
  if (OnKeyDown != NULL)
    {
      // EventFunction
      Push (EventsFunction,OnKeyDown);

      // Message Type
      Push (MessageType,"SEL_KEYPRESS");

      // MessageID
      char MsgID [512];
      sprintf (MsgID,"ID_%s",Stack -> Name);
      Push (MessageID,MsgID);
    }

  // Event - OnKeyUp
  if (OnKeyUp != NULL)
    {
      // EventFunction
      Push (EventsFunction,OnKeyUp);

      // Message Type
      Push (MessageType,"SEL_KEYRELEASE");

      // MessageID
      char MsgID [512];
      sprintf (MsgID,"ID_%s",Stack -> Name);
      Push (MessageID,MsgID);
    }

  // clear
  delete [] Caption;
  delete [] FontColor;
  delete [] Hint;
  delete [] FontCharset;
  delete [] FontName;
  delete [] Enabled;
  delete [] Cursor;
  delete [] GlyphData;
  delete [] Layout;
  delete [] Flat;
  delete [] OnClick;
  delete [] OnKeyDown;
  delete [] OnKeyUp;
}