//---------------------------------------------------------------------------
// TStaticText Class
//---------------------------------------------------------------------------

/*
Supported properties:

  Left
  Top
  Width
  Height
  Caption
  Color
  Hint
  Layout
  Alignment
  Font.Color
  Font.Charset
  Font.Height
  Font.Name
  Font.Style
  Align
  Cursor
  Enabled
  BorderStyle

Supported events:
  OnClick
  OnMouseEnter
  OnMouseLeave
  OnMouseDown
  OnMouseUp
*/

/*
  Known problems:
  - BorderStyle: sbsSingle
*/


void DFMParser::TStaticTextClass ()
{
  // attributes
  int  Left;
  int  Top;
  int  Width;
  int  Height;
  char *Caption = NULL;
  char *FontColor = NULL;
  char *Color = NULL;
  char *Hint = NULL;
  char *ShowHint = NULL;
  char *Layout = NULL;
  char *Alignment = NULL;
  char *FontCharset = NULL;
  int FontHeight = 0;
  char *FontName = NULL;
  StackElement *FontStyle = NULL;
  char *Align = NULL;
  char *Cursor = NULL;
  char *Enabled = NULL;
  char *BorderStyle = NULL;

  // events
  char *OnClick = NULL;
  char *OnMouseEnter = NULL;
  char *OnMouseLeave = NULL;
  char *OnMouseDown = NULL;
  char *OnMouseUp = NULL;

  // read words...
  bool Error;
  char *Name = ReadString (Error);
  while (strcmp (Name,"object") != 0 && strcmp (Name,"end") != 0)
    {
      // Left
      ReadDigitProperty (Name,"Left",Left);

      // Top
      ReadDigitProperty (Name,"Top",Top);

      // Height
      ReadDigitProperty (Name,"Height",Height);

      // Width
      ReadDigitProperty (Name,"Width",Width);

      // Caption
      ReadQuotationProperty (Name,"Caption",Caption);

      // Color
      ReadStringProperty (Name,"Color",Color);

      // Hint
      ReadQuotationProperty (Name,"Hint",Hint);

      // ShowHint
      ReadStringProperty (Name,"ShowHint",ShowHint);

      // Layout
      ReadStringProperty (Name,"Layout",Layout);

      // Alignment
      ReadStringProperty (Name,"Alignment",Alignment);

      // Font Charset
      ReadStringProperty (Name,"Font.Charset",FontCharset);

      // Font Color
      ReadStringProperty (Name,"Font.Color",FontColor);

      // Font Height
      ReadDigitProperty (Name,"Font.Height",FontHeight);

      // Font Name
      ReadQuotationProperty (Name,"Font.Name",FontName);

      // Font Style
      ReadSetProperty (Name,"Font.Style",FontStyle);

      // Align
      ReadStringProperty (Name,"Align",Align);

      // Cursor
      ReadStringProperty (Name,"Cursor",Cursor);

      // Enabled
      ReadStringProperty (Name,"Enabled",Enabled);

      // BorderStyle
      ReadStringProperty (Name,"BorderStyle",BorderStyle);

      // OnClick
      ReadStringProperty (Name,"OnClick",OnClick);

      // OnMouseEnter
      ReadStringProperty (Name,"OnMouseEnter",OnMouseEnter);

      // OnMouseLeave
      ReadStringProperty (Name,"OnMouseLeave",OnMouseLeave);

      // OnMouseDown
      ReadStringProperty (Name,"OnMouseDown",OnMouseDown);

      // OnMouseUp
      ReadStringProperty (Name,"OnMouseUp",OnMouseUp);

      // clear old and read new word ...
      delete [] Name;
      Name = ReadString (Error);
    }
  delete [] Name;

  // *.h file
  fprintf (hfile,"  FXLabel *%s;\n",Stack -> Name);

  // *.cpp file
  fprintf (cppfile,"  %s = new FXLabel (%s,",Stack -> Name,Stack -> Next -> Name);

  // Caption
  if (Caption != NULL)
    {
      fprintf (cppfile,"\"%s",Caption);
    }

  // Hint
  if (Hint != NULL)
    {
      if (ShowHint == NULL || strcmp (ShowHint,"True") == 0)
        {
          fprintf (cppfile,"\\t%s",Hint);
        }
    }

  fprintf (cppfile,"\",NULL,");

  // Align
  fprintf (cppfile,"%s",GetAlign (Align));

  // Layout
  if (Align != NULL && strcmp (Align,"alNone") != 0)
    {
      fprintf (cppfile," | %s",GetLayout (Layout));
    }

  // Alignment
  if (Align != NULL && strcmp (Align,"alNone") != 0)
    {
      fprintf (cppfile," | %s",GetAlignment (Alignment));
    }

  // BorderStyle
  if (BorderStyle != NULL && strcmp (BorderStyle,"sbsSunken") == 0)
    {
      fprintf (cppfile," | FRAME_SUNKEN");
    }

  // Left, Top, Width, Height
  fprintf (cppfile,",%i,%i,%i,%i,0,0,0,0);\n",Left,Top,Width,Height);

  // Font Color
  if (FontColor != NULL && strlen (GetColor (FontColor)) > 0)
    {
      fprintf (cppfile,"  %s -> setTextColor (%s);\n",Stack -> Name,GetColor (FontColor));
    }

  // Color
  if (Color != NULL && strlen (GetColor (Color)) > 0)
    {
      fprintf (cppfile,"  %s -> setBackColor (%s);\n",Stack -> Name,GetColor (Color));
    }

  // Font
  if (FontHeight != 0 && FontName != NULL)
    {
      fprintf (cppfile,"  %s -> setFont (new FXFont (getApp(),\"%s\",%i",Stack -> Name,FontName,(int)((-FontHeight * 72) / 96));

      bool Bold = false;
      bool Italic = false;
      while (!Empty (FontStyle))
        {
          if (strcmp (FontStyle -> Name,"fsBold") == 0)
            Bold = true;
          else
          if (strcmp (FontStyle -> Name,"fsItalic") == 0)
            Italic = true;
          Pop (FontStyle);
        }

      // Bold
      if (Bold == true)
        fprintf (cppfile,",FONTWEIGHT_BOLD");
      else
        fprintf (cppfile,",FONTWEIGHT_NORMAL");

      // Italic
      if (Italic == true)
        fprintf (cppfile,",FONTSLANT_ITALIC");
      else
        fprintf (cppfile,",FONTSLANT_REGULAR");

      // Charset
      if (FontCharset != NULL)
        fprintf (cppfile,",%s));\n",GetFontCharset (FontCharset));
      else
        fprintf (cppfile,"));\n");
    }

  // Cursor
  if (Cursor != NULL && strlen (GetCursor (Cursor)) > 0)
    {
      fprintf (cppfile,"  %s -> setDefaultCursor(getApp () -> getDefaultCursor(%s));\n",Stack -> Name,GetCursor (Cursor));
    }

  // Enabled
  if (Enabled != NULL && strcmp (Enabled,"False") == 0)
    {
      fprintf (cppfile,"  %s -> disable ();\n",Stack -> Name);
    }

  // Event - OnClick
  if (OnClick != NULL)
    {
      // EventFunction
      Push (EventsFunction,OnClick);

      // Message Type
      Push (MessageType,"SEL_LEFTBUTTONPRESS");

      // MessageID
      char MsgID [512];
      sprintf (MsgID,"ID_%s",Stack -> Name);
      Push (MessageID,MsgID);
    }

  // OnMouseEnter
  if (OnMouseEnter != NULL)
    {
      // EventFunction
      Push (EventsFunction,OnMouseEnter);

      // Message Type
      Push (MessageType,"SEL_ENTER");

      // MessageID
      char MsgID [512];
      sprintf (MsgID,"ID_%s",Stack -> Name);
      Push (MessageID,MsgID);
    }

  // OnMouseLeave
  if (OnMouseLeave != NULL)
    {
      // EventFunction
      Push (EventsFunction,OnMouseLeave);

      // Message Type
      Push (MessageType,"SEL_LEAVE");

      // MessageID
      char MsgID [512];
      sprintf (MsgID,"ID_%s",Stack -> Name);
      Push (MessageID,MsgID);
    }

  // Event - OnMouseDown, it's simulated by SEL_LEFTBUTTONPRESS,
  // SEL_MIDDLEBUTTONPRESS and SEL_RIGHTBUTTONPRESS

  if (OnMouseDown != NULL)
    {
      // Event functions
      Push (EventsFunction,OnMouseDown);
      Push (EventsFunction,OnMouseDown);
      Push (EventsFunction,OnMouseDown);

      // Message types
      Push (MessageType,"SEL_LEFTBUTTONPRESS");
      Push (MessageType,"SEL_MIDDLEBUTTONPRESS");
      Push (MessageType,"SEL_RIGHTBUTTONPRESS");

      // MessageIDs
      char MsgID [512];
      sprintf (MsgID,"ID_%s",Stack -> Name);
      Push (MessageID,MsgID);
      Push (MessageID,MsgID);
      Push (MessageID,MsgID);
    }

  // Event - OnMouseUp, it's simulated by SEL_LEFTBUTTONRELEASE,
  // SEL_MIDDLEBUTTONRELEASE and SEL_RIGHTBUTTONRELEASE

  if (OnMouseUp != NULL)

    {
      // Event functions
      Push (EventsFunction,OnMouseUp);
      Push (EventsFunction,OnMouseUp);
      Push (EventsFunction,OnMouseUp);

      // Message types
      Push (MessageType,"SEL_LEFTBUTTONRELEASE");
      Push (MessageType,"SEL_MIDDLEBUTTONRELEASE");
      Push (MessageType,"SEL_RIGHTBUTTONRELEASE");

      // MessageIDs
      char MsgID [512];
      sprintf (MsgID,"ID_%s",Stack -> Name);
      Push (MessageID,MsgID);
      Push (MessageID,MsgID);
      Push (MessageID,MsgID);
    }

  // any Event
  if (OnClick != NULL || OnMouseEnter != NULL || OnMouseLeave != NULL ||
      OnMouseDown != NULL || OnMouseUp != NULL)
    {
      // MessageID
      char MsgID [512];
      sprintf (MsgID,"ID_%s",Stack -> Name);

      // select Target and Selector
      fprintf (cppfile,"  %s -> setTarget (this);\n",Stack -> Name);
      fprintf (cppfile,"  %s -> setSelector (%s);\n",Stack -> Name,MsgID);
    }

  // one empty line
  fprintf (cppfile,"\n");

  // clear
  delete [] Caption;
  delete [] FontColor;
  delete [] Color;
  delete [] Hint;
  delete [] ShowHint;
  delete [] Layout;
  delete [] Alignment;
  delete [] FontCharset;
  delete [] FontName;
  delete [] Align;
  delete [] Cursor;
  delete [] Enabled;
  delete [] BorderStyle;
  delete [] OnClick;
  delete [] OnMouseEnter;
  delete [] OnMouseLeave;
  delete [] OnMouseDown;
  delete [] OnMouseUp;
}
